#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# System modules
import requests
from astropy.time import Time
from urllib.parse import urlencode
from datetime import datetime

# Custom modules
from utils.logger import log
from devices import check


class DlinkDCSCamera:
    def __init__(self, url):
        self.url = url
        self.addr = self.url
        self.timeout = 3
        self.error = []

#METODI PRINCIPALI
    @check.request_errors
    def get(self, method, params=[]):
        res = requests.get(f"{self.addr}/{method}.cgi",
                           params=params,
                           timeout=self.timeout)
        #add error check
        return res

    @check.request_errors
    def put(self, method, params={}):
        res = requests.get(f"{self.addr}/{method}.cgi",
                           params=urlencode(params),
                           timeout=self.timeout)
        value = self.unmarshal_response(res.content.decode('utf-8'))
        #add error check
        return value

    def unmarshal_response(self, response):
        """Unmarshal the multiline key value pair response."""
        _obj = {}
        for line in response.splitlines():
            # ignore blank lines and xml <result> block
            if line != '' and not line.startswith('<'):
                _keyvalue = line.strip().split("=")
                _obj[_keyvalue[0]] = _keyvalue[1]
        return _obj

#METODI SECONDARI
    def time_to_string(self, time):
        """Convert a datetime into the HH:MM:SS string format."""
        return datetime.strftime(time, '%H:%M:%S')

##STATUS
    @property
    def description(self):
        """Get IP Camera Information."""
        res = self.get('common/info')
        return self.unmarshal_response(res.content.decode('utf-8'))

    @property
    def datetime(self):
        """Get IP Camera Data Time settings."""
        return self.get('datetime')

    @property
    def version(self):
        """Get IP Camera CGI version."""
        return self.get('cgiversion')

    @property
    def inetwork(self):
        """Get the IP Camera Network information."""
        return self.get('inetwork')

    @property
    def isystem(self):
        """Get the IP Camera System information."""
        return self.get('isystem')

    @property
    def iwireless(self):
        """Get the IP Camera Wireless information."""
        return self.get('iwireless')

    @property
    def network(self):
        """Get the IP Camera Network settings."""
        return self.get('network')

    @property
    def user(self):
        """Get the IP Camera user setttings."""
        return self.get('user')

    @property
    def user_list(self):
        """Get the list of IP Camera users."""
        return self.get('userlist')

##MAIN SETTINGS
    @property
    def iimage(self):
        """Get the IP Camera Image information."""
        return self.get('iimage')

    @property
    def image(self):
        """Get the IP Camera Image information."""
        return self.get('image')

    @property
    def upload(self):
        """Get the IP Camera FTP Upload settings."""
        return self.get('upload')

    @property
    def stream_info(self):
        """Get the IP Camera Video stream info."""
        return self.get('config/stream_info')

    @property
    def ptz(self):
        """Get the IP Camera Network Pan Tilt Zoom."""
        return self.get('config/ptz_move')

    @property
    def is_moving(self):
        """Get the IP Camera Motion Detection settings."""
        return self.get('motion')

##SECONDARY SETTINGS
    @property
    def daynight(self):
        """Get the IP Camera Day Night Mode settings."""
        return self.get('daynight')

    @property
    def email(self):
        """Get the IP Camera Email notification settings."""
        return self.get('email')

    @property
    def ptzpresets(self):
        """Get the IP Camera Network Pan Tilt Zoom Preset List"""
        return self.get('config/ptz_preset_list')

    @property
    def sound_detection(self):
        """Get the IP Camera Sound Detection settings."""
        return self.get('sdbdetection')

#ALLINFO
    @property
    def all(self):
        res=0
        return res
